package com.flyfish.framework.service.impl;

import com.flyfish.framework.domain.base.Domain;
import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.repository.DefaultReactiveRepository;
import com.flyfish.framework.service.BaseReactiveService;
import com.flyfish.framework.utils.Assert;
import com.flyfish.framework.utils.CopyUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageImpl;
import org.springframework.data.domain.Pageable;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 基础的Service，继承常用的全部Crud操作，基于JPARepository
 * TODO 此方法在更新和查询存在不完善，后续完善底层支持，用于性能优化
 *
 * @param <T> 实体泛型
 */
public class BaseReactiveServiceImpl<T extends Domain> implements BaseReactiveService<T> {

    @Autowired
    protected DefaultReactiveRepository<T> repository;

    /**
     * 查询
     *
     * @param entity 实体本身
     * @return 结果
     */
    @Override
    public Mono<T> getOne(T entity) {
        return repository.findOne(Example.of(entity));
    }

    /**
     * 查询
     *
     * @param query 查询模型
     * @return 结果
     */
    @Override
    public Mono<T> getOne(Qo<T> query) {
        return Mono.empty();
    }

    /**
     * 通过Id查询
     *
     * @param id 主键
     * @return 结果
     */
    @Override
    public Mono<T> getById(String id) {
        return repository.findById(id);
    }

    /**
     * 根据ID集合来查询
     *
     * @param ids id列表
     * @return 结果
     */
    @Override
    public Flux<T> getByIds(List<String> ids) {
        return repository.findAllById(ids);
    }

    /**
     * 查询列表
     *
     * @param entity 实体本身
     * @return 结果
     */
    @Override
    public Flux<T> getList(T entity) {
        return repository.findAll(Example.of(entity));
    }

    /**
     * 查询列表
     *
     * @param query 查询
     * @return 结果
     */
    @Override
    public Flux<T> getList(Qo<T> query) {
        return Flux.empty();
    }

    /**
     * 分页查询列表
     *
     * @param entity   实体本身
     * @param pageable 分页
     * @return 结果
     */
    @Override
    public Mono<Page<T>> getPageList(T entity, Pageable pageable) {
        return repository.findAll(Example.of(entity), pageable.getSort())
                .buffer(pageable.getPageSize(), (int) pageable.getOffset())
                .reduce(new PageImpl<>(new ArrayList<>()), (result, item) -> {
                    List<T> list = result.getContent();
                    list.addAll(item);
                    return new PageImpl<>(list, pageable, result.getTotalElements());
                });
    }

    /**
     * 分页查询列表
     *
     * @param query 查询实体
     * @return 结果
     */
    @Override
    public Mono<Page<T>> getPageList(Qo<T> query) {
        return Mono.empty();
    }

    /**
     * 获取所有对象
     *
     * @return 结果
     */
    @Override
    public Flux<T> getAll() {
        return repository.findAll();
    }

    /**
     * 查询总记录数
     *
     * @return 总数
     */
    @Override
    public Mono<Long> countAll() {
        return repository.count();
    }

    /**
     * 查询总记录数
     *
     * @param entity 通过实体查询个数
     * @return 结果
     */
    @Override
    public Mono<Long> count(T entity) {
        return repository.count(Example.of(entity));
    }

    /**
     * 插入新记录
     *
     * @param entity 实体
     */
    @Override
    public Mono<T> create(T entity) {
        return repository.insert(entity);
    }

    /**
     * 插入 不插入null字段
     *
     * @param entity 实体
     */
    @Override
    public Mono<T> createSelective(T entity) {
        return repository.insert(entity);
    }

    /**
     * 删除
     *
     * @param entity 实体
     */
    @Override
    public Mono<Void> delete(T entity) {
        return repository.delete(entity);
    }

    /**
     * 根据Id删除
     *
     * @param id 主键
     */
    @Override
    public Mono<Void> deleteById(String id) {
        return repository.deleteById(id);
    }

    /**
     * 根据id更新
     *
     * @param entity 实体
     */
    @Override
    public Mono<T> updateById(T entity) {
        return repository.save(entity);
    }

    /**
     * 不update null
     *
     * @param entity 实体
     */
    @Override
    public Mono<T> updateSelectiveById(T entity) {
        Assert.hasText(entity.getId(), "更新的主键不可为空！");
        Mono<T> saved = repository.findById(entity.getId());
//        Assert.isTrue(saved.isPresent(), "要更新的信息不存在！");
        return repository.saveAll(saved.filter(Objects::nonNull)
                .map(t -> CopyUtils.copyProps(entity, t)).flux()).single();
    }

    /**
     * 根据ID集合批量删除
     *
     * @param ids id结合
     */
    @Override
    public Mono<Void> deleteBatchByIds(List<String> ids) {
        return Flux.fromIterable(ids)
                .flatMap(t -> repository.deleteById(t))
                .single();
    }

    /**
     * 批量更新
     *
     * @param entities 要批量更新的集合
     */
    @Override
    public Flux<T> updateBatch(List<T> entities) {
//        Assert.notEmpty(entities, "数据不可为空！");
//        entities.forEach(item -> Assert.hasText(item.getId(), "每个对象的id必须附带！"));
//        // 过滤不存在的记录
//        Map<String, T> savedMap = ((List<T>) repository.findAllById(entities.stream()
//                .map(Domain::getId).collect(Collectors.toList()))).stream()
//                .collect(Collectors.toMap(Domain::getId, t -> t));
//        List<T> updating = entities.stream()
//                .filter(t -> savedMap.containsKey(t.getId()))
//                .map(t -> CopyUtils.copyProps(savedMap.get(t.getId()), t))
//                .collect(Collectors.toList());
        return repository.saveAll(entities);
    }
}

